#ifndef __TYPES_H
#define __TYPES_H

#include <stdint.h>
typedef	uint64_t u64;
typedef uint32_t u32;
typedef uint16_t u16;
typedef uint8_t u8;
typedef int64_t s64;
typedef int32_t s32;
typedef int16_t s16;
typedef int8_t  s8;

#endif	// __TYPES_H
