void ExtractFiles(char *ndsfilename);
void ExtractOverlayFiles();
void Extract(char *outfilename, bool indirect_offset, unsigned int offset, bool indirect_size, unsigned size, bool with_footer = false);
